/* KEYTEST1.C
 *
 * By Gary Neal, Jr. -- garyneal@mailcity.com
 * http://www.geocities.com/SiliconValley/Park/7113/
 *
 * First test of the Low Level Keyboard Handler.
 * This test will use it in a way that most games will use it.
 */

#include <dos.h>       /* Support for low level dos functions. */
#include <stdio.h>     /* Support for 'fputs' function. */
#include "keyboard.h"  /* Low Level Keyboard Handler functions. */
#include "boolean.h"   /* Data type 'bool' support */

int main(void)
{
    int X, Y;                    /* Pixel location */
    union REGS regs;             /* Register structure varriable */

    regs.x.ax = 0x0013;          /* Request video mode 13h */
    int86(0x10, &regs, &regs);
    regs.h.ah = 0x0F;            /* Check to see if mode was changed */
    int86(0x10, &regs, &regs);
    if (regs.h.al != 0x13) {
        fputs("VGA 320 x 200 x 256 not supported", stderr);
        return 1;
    }

    /* Set Pixel position. */
    X = 160;
    Y = 100;

    /* Install low level keyboard handler. *
     * Do not chain BIOS.                  *
     * Do not use a callback function.     */
    InstallKeyboardHandler(false, 0);

    /* Continue until ESC key pressed or a hard abort was encountered. */
    while (!KeyPressed(KEY_ESC) && !KeyHardAbort()) {
        pokeb(0xA000, Y * 320 + X, 0);         /* Erase pixel */

        /* Update pixel position according to what keys are pressed. */
        if (KeyPressed(KEY_UP)) {
            if (--Y < 0) Y = 0;
        } else if (KeyPressed(KEY_DOWN)) {
            if (++Y > 199) Y = 199;
        }

        if (KeyPressed(KEY_LEFT)) {
            if (--X < 0) X = 0;
        } else if (KeyPressed(KEY_RIGHT)) {
            if (++X > 319) X = 319;
        }

        pokeb(0xA000, Y * 320 + X, 15);        /* Set pixel */
        delay(10);                             /* Delay 10ms (Borland only) */
    }

    /* Remove Low level Keyboard Handler */
    RemoveKeyboardHandler();

    /* Restore text mode */
    regs.x.ax = 0x0003;
    int86(0x10, &regs, &regs);

    return 0;
}
