/* KEYTEST4.C
 *
 * By Gary Neal, Jr. -- garyneal@mailcity.com
 * http://www.geocities.com/SiliconValley/Park/7113/
 *
 * This program illustrates how to specify a callback function for the
 * low level keyboard handler.
 *
 * The callback function will be called in an interrupt context.
 * (Must be very short and quick, cannot make calls to DOS, etc..)
 */

#include <dos.h>
#include <stdio.h>
#include "keyboard.h"

/* Counts the number of calls to the low level keyboard handler. */
volatile long keyDriverHits = 0;

/* Callback function used by the low level keyboard handler. */
void far CountKeyHits(void)
{
    ++keyDriverHits;
}

int main(void)
{
    int X, Y;                   /* Pixel location. */
    union REGS regs;            /* Register variable structure. */

    regs.x.ax = 0x0013;         /* Request video mode 13h */
    int86(0x10, &regs, &regs);
    regs.h.ah = 0x0F;           /* Check to see if mode was changed. */
    int86(0x10, &regs, &regs);
    if (regs.h.al != 0x13) {
        fputs("VGA 320 x 200 x 256 not supported", stderr);
        return 1;
    }

    /* Set the pixel location. */
    X = 160;
    Y = 100;

    /* Install low level keyboard handler. *
     *     Don't chain BIOS.               *
     *     Specify a callback function.    */
    InstallKeyboardHandler(false, CountKeyHits);

    /* Continue until ESC key pressed or a hard abort encountered. */
    while (!KeyPressed(KEY_ESC) && !KeyHardAbort()) {
        pokeb(0xA000, Y * 320 + X, 0);      /* Erase pixel. */

        /* Update pixel's position according to what keys are pressed. */
        if (KeyPressed(KEY_UP)) {
            if (--Y < 0) Y = 0;
        } else if (KeyPressed(KEY_DOWN)) {
            if (++Y > 199) Y = 199;
        }

        if (KeyPressed(KEY_LEFT)) {
            if (--X < 0) X = 0;
        } else if (KeyPressed(KEY_RIGHT)) {
            if (++X > 319) X = 319;
        }

        pokeb(0xA000, Y * 320 + X, 15);     /* Redraw pixel. */
        delay(10);                          /* Delay 10ms (Borland only) */
    }

    /* Remove low level keyboard handler. */
    RemoveKeyboardHandler();

    /* Restore text mode. */
    regs.x.ax = 0x0003;
    int86(0x10, &regs, &regs);

    /* Display count of the calls to low level keyboard handler. */
    printf("Low level keyboard handler was called %ld times.", keyDriverHits);

    return 0;
}
